% Function: InvertedBatConvertZetaWnToGains
% Purpose:  Calculate feedback control gains for classic inverted pendulum on a cart
% Authors:  Paul Mitiguy and Tony Rivera
%
% Notes:    The linearized equations of motion for this system were generated by
%           the symbolic manipulation program Autolev - http://www.autolev.com
%           The visualization of a (SolidWorks) hand balancing a baseball bat
%           is demonstrated in visualNastran 4D - http://vnDesktop.com
% Inputs:
% zeta:           A positive damping ratio corresponds to a stable solution
%                 A negative damping ratio corresponds to an unstable solution
%                 zeta = 0      corresponds to a purely oscillatory solution
%                 0 < zeta < 1  corresponds to a decaying oscillatory solution
%                 zeta = 1      corresponds to a critically damped solution (no oscillations)
%                 zeta > 1      corresponds to an overdamped solution (no oscillations and slow)
% Wn:             Natural frequency.  A larger Wn corresponds to a faster response
% angleIndicator: 0=0 radians,  1=pi radians all other numbers are converted to 0 or pi by
%                 nTheta = floor( mod(fabs(angleIndicator),2) ) * pi;
% g:              Gravitational constant, normally 9.81
% L:              Distance from the hand to the center of mass of the bat
% mA:             Mass of the hand (cart)
% mB:             Mass of the baseball bat (inverted pendulum)
% J:              Central moment of inertia of the baseball bat
%
% Output:  A column matrix of the feedback control gains, i.e., [k1; k2; k3; k4]
%          The state matrix is assumed to be [x, theta, x', theta']
%
function returnValue = InvertedBatConvertZetaWnToGains( zeta, Wn, angleIndicator, g, L, mA, mB, J )

% Calculate pole locations
Wd = Wn * sqrt( 1 - zeta^2 );
L1 = -Wn*zeta + i*Wd;    % pole(1)
L2 = -Wn*zeta - i*Wd;    % pole(2)
L3 = -Wn*zeta + i*Wd;    % pole(3)
L4 = -Wn*zeta - i*Wd;    % pole(4)

% Write the desired pole location in a polynomial form, i.e.,
% (L-L1)*(L-L2)*(L-L3)*(L-L4) = z1 + z2*L + z3*L^2/2! + z4*L^3/3! + L^4
z1 = L1*L2*L3*L4;
z2 = -L1*L2*L3 - L1*L2*L4 - L1*L3*L4 - L2*L3*L4;
z3 = 2*L1*L2 + 2*L1*L3 + 2*L1*L4 + 2*L2*L3 + 2*L2*L4 + 2*L3*L4;
z4 = -6*L1 - 6*L2 - 6*L3 - 6*L4;

% Calculate the nominal angle from the angleIndicator (up=0, down=1)
nTheta = floor( mod(abs(angleIndicator),2) ) * pi;

% Calculate feedback gains based on pole locations
% Effectively, this carries out the  place()  function in Matlab
% These numbers correspond to the linearized plant in Autolev

z5 = g*L^2*mB^2/(J*(mA+mB)+mA*mB*L^2);
z6 = cos(nTheta);
z7 = g*L*mB*(mA+mB)*z6/(J*(mA+mB)+mA*mB*L^2);
z8 = (J+mB*L^2)/(J*(mA+mB)+mA*mB*L^2);
z9 = L*mB*z6/(J*(mA+mB)+mA*mB*L^2);

k1 = z1/(z5*z9-z7*z8);
k2 = -0.5*(z3+2*z7-2*k1*z8)/z9;
k3 = z2/(z5*z9-z7*z8);
k4 = -0.1666667*(z4-6*k3*z8)/z9;

% There may be round-off errors due to multiplying complex numbers
% that produce tiny imaginary parts.  Use the real function to
% get rid of the tiny imaginary parts.  Also, the Autolev analyis
% has the angle theta defined in an opposite way then the 
% visualNastran 4D file, so negate the return values associated
% with theta and theta'
returnValue = -[ real(k1); -real(k2); real(k3); -real(k4) ];
